@@ -77,8 +77,39 @@ _gap = [_np.nan, _np.nan]   # To leave a break in the plot
 _rh = 0.25      # Resistor height
 _rw = 1.0 / 6   # Full (inner) length of resistor is 1.0 data unit
 
-# Basic elements
+# Drawing Functions
+def _cycloid(loops=4, ofst=(0,0), a=.06, b=.19, norm=True, vertical=False, flip=False):
+    ''' Generate a prolate cycloid (inductor spiral) that
+        will always start and end at y=0.
+        loops = Number of loops
+        a, b = parameters. b>a for prolate (loopy) cycloid
+        norm = Normalize the length to 1 [True/False]
+        vertical, flip = Control the orientation of cycloid
+    '''
+    yint = _np.arccos(a/b) # y-intercept
+    t = _np.linspace(yint, 2*(loops+1)*_np.pi-yint, num=loops*50)
+    x = a*t - b*_np.sin(t)
+    y = a   - b*_np.cos(t)
+    x = x - x[0] # Shift to start at 0,0
 
+    if norm:
+        x = x / (x[-1]-x[0])      # Normalize length to 1
+    
+    if flip:
+        y = -y
+    
+    y = y * (max(y)-min(y))/(_rh) # Normalize to resistor width
+    
+    if vertical:
+        x, y = y, x
+
+    x = x + ofst[0]
+    y = y + ofst[1]
+    
+    path = _np.transpose(_np.vstack((x,y)))
+    return path
+
+# Basic elements
 RES = {
     'name'  : 'RES',
     'paths' : [ [[0,0],[0.5*_rw,_rh],[1.5*_rw,-_rh],[2.5*_rw,_rh],[3.5*_rw,-_rh],[4.5*_rw,_rh],[5.5*_rw,-_rh],[6*_rw,0]] ],
@@ -87,7 +118,12 @@ RES = {
 RES_VAR = {
     'name'  : 'RES_VAR',
     'base'  : RES,
-    'paths' : [ [[.5*_rw,-_rh*3],[5.5*_rw,_rh*3],[4.*_rw,2.5*_rh],_gap,[5.5*_rw,_rh*3],[5.8*_rw,1.9*_rh]] ]
+    'shapes' : [ { 'shape' : 'arrow',
+                   'start' : [1.5*_rw,-_rh*2],
+                   'end'   : [4.5*_rw,_rw*3.5],
+                   'headwidth' : .12,
+                   'headlength' : .2 } ],
+    'paths' : [ [[1.5*_rw,-_rh*2],[4.5*_rw,_rw*3.5]] ]
      }
 
 RBOX = {
@@ -107,21 +143,22 @@ POT = {
     'anchors' : { 'tap' : [_rw*3,_rw*1.5] }
      }
 
-_cap_gap = 0.2
+
+_cap_gap = 0.18
 CAP = {   # Straight capacitor
     'name'  : 'CAP',
-    'paths'   : [ [[0,0],[0,_rh],[0,-_rh],_gap,[_cap_gap,_rh],[_cap_gap,-_rh],[_cap_gap,0]] ],
+    'paths'   : [ [[0,0],_gap,[0,_rh],[0,-_rh],_gap,[_cap_gap,_rh],[_cap_gap,-_rh],_gap,[_cap_gap,0]] ],
     }
 
 CAP_P = {   # Polarized
     'name'  : 'CAP_P',
     'base'  : CAP,
-    'labels' : [ {'label':'+', 'pos':[-_cap_gap,_cap_gap]} ]
+    'labels' : [ {'label':'+', 'pos':[-_cap_gap*1.2,_cap_gap]} ]
     }
 
 CAP2 = {  # Curved capacitor
     'name'  : 'CAP2',
-    'paths'   : [ [[0,0],[0,_rh],[0,-_rh],_gap,[_cap_gap,0]] ],
+    'paths'   : [ [[0,0],_gap,[0,_rh],[0,-_rh],_gap,[_cap_gap,0]] ],
     'shapes'  : [ { 'shape'  : 'arc',
                     'center' :[(_cap_gap*1.5),0],
                     'theta1' : 120,
@@ -133,9 +170,17 @@ CAP2 = {  # Curved capacitor
 CAP2_P = {   # Polarized
     'name'  : 'CAP2_P',
     'base'  : CAP2,
-    'labels' : [ {'label':'+', 'pos':[-_cap_gap,_cap_gap]} ]
+    'labels' : [ {'label':'+', 'pos':[-_cap_gap*1.2,_cap_gap]} ]
     }
 
+_cap_gap = 0.2
+XTAL = {   # Crystal
+    'name'  : 'XTAL',
+    'paths'   : [ [[0,0],_gap,[0,_rh],[0,-_rh],_gap,
+                   [_cap_gap/2,_rh],[_cap_gap/2,-_rh],[_cap_gap*1.5,-_rh],[_cap_gap*1.5,_rh],[_cap_gap/2,_rh],_gap,
+                   [_cap_gap*2,_rh],[_cap_gap*2,-_rh],_gap,[_cap_gap*2,0]],
+                ],
+    }
 
 DIODE = {
     'name'  : 'DIODE',
@@ -154,22 +199,110 @@ DIODE_F = {
                     'fill' : True } ]
     }
 
+_sd = .1
+SCHOTTKY = {  # Schottky diode
+    'name' : 'SCHOTTKY',
+    'base' : DIODE,
+    'paths' : [ [[_rh*1.4,_rh], [_rh*1.4-_sd,_rh],  [_rh*1.4-_sd,_rh-_sd]],
+                [[_rh*1.4,-_rh], [_rh*1.4+_sd,-_rh], [_rh*1.4+_sd,-_rh+_sd]] ]
+}
+
+SCHOTTKY_F = {
+    'name' : 'SCHOTTKY_F',
+    'base' : DIODE_F,
+    'paths' : [ [[_rh*1.4,_rh], [_rh*1.4-_sd,_rh],  [_rh*1.4-_sd,_rh-_sd]],
+                [[_rh*1.4,-_rh], [_rh*1.4+_sd,-_rh], [_rh*1.4+_sd,-_rh+_sd]] ]
+}
+
+ZENER = {  # Zener diode
+    'name' : 'ZENER',
+    'base' : DIODE,
+    'paths' : [ [[_rh*1.4,_rh],  [_rh*1.4+_sd,_rh+_sd]],
+                [[_rh*1.4,-_rh], [_rh*1.4-_sd,-_rh-_sd]] ]
+}
+
+ZENER_F = {
+    'name' : 'ZENER_F',
+    'base' : DIODE_F,
+    'paths' : [ [[_rh*1.4,_rh],  [_rh*1.4+_sd,_rh+_sd]],
+                [[_rh*1.4,-_rh], [_rh*1.4-_sd,-_rh-_sd]] ]
+}
+
 LED = {
     'name':'LED',
     'base':DIODE,
+    'paths' : [[[_rh,_rh*1.5],[_rh*2,_rh*3.25]]],  # Duplicate arrow with a path to work around matplotlib autoscale bug.
     'shapes':[ { 'shape':'arrow',
                  'start': [_rh,_rh*1.5],
                  'end'  : [_rh*2,_rh*3.25],
-                   'headwidth' : .17,
+                   'headwidth' : .12,
                    'headlength' : .2 },
-
                { 'shape':'arrow',
                  'start': [_rh*.1,_rh*1.5],
                  'end'  : [_rh*1.1,_rh*3.25],
-                   'headwidth' : .17,
+                   'headwidth' : .12,
                    'headlength' : .2 }, ]
     }
 
+# LED with squiggle light
+x = _np.linspace(-1,1)
+y = -x*(x-.7)*(x+.7)/2 + _rh*2.5
+x = _np.linspace(_rh*.75,_rh*1.25)
+theta = 20
+c = _np.cos( _np.radians(theta) )
+s = _np.sin( _np.radians(theta) )
+m = _np.array( [[c,s],[-s,c]] )
+p = _np.transpose(_np.vstack((x,y)))
+p = _np.dot( p, m )
+p2 = _np.transpose(_np.vstack((x-.2,y)))
+p2 = _np.dot( p2, m )
+LED2 = {
+    'name'   : 'LED2',
+    'base'   : DIODE,
+    'paths'  : [p,p2],
+    'shapes' : [ {'shape':'arrow',
+                  'start' : p[1],
+                  'end'   : p[0],
+                  'headwidth' : .07,
+                  'headlength' : .08
+                 },
+                 {'shape':'arrow',
+                  'start' : p2[1],
+                  'end'   : p2[0],
+                  'headwidth' : .07,
+                  'headlength' : .08
+                 } ]
+    }
+
+
+_mr = 0.2
+MEMRISTOR = { 
+    'name'  : 'MEMRISTOR',
+    'paths' : [ [[0,_mr*1.25],[_mr*5,_mr*1.25],[_mr*5,_mr*-1.25],[0,_mr*-1.25],[0,_mr*1.25]],
+                [[0,0],[_mr,0],[_mr,-_mr*.75],[_mr*2,-_mr*.75],[_mr*2,_mr*.75],[_mr*3,_mr*.75],[_mr*3,-_mr*.75],[_mr*4,-_mr*.75],[_mr*4,0],[_mr*5,0]],
+              ],
+    'shapes' : [ {'shape':'poly', 
+                  'xy' : [[0,_mr*1.25],[0,-_mr*1.25],[_mr/2,-_mr*1.25],[_mr/2,_mr*1.25] ],
+                  'fill' : True
+                 }
+               ]
+    }
+
+_mrv = .25
+MEMRISTOR2 = { 
+    'name'  : 'MEMRISTOR2',
+    'paths' : [ [[0,0],[0,_mrv],[_mr,_mrv],[_mr,-_mrv],[_mr*2,-_mrv],[_mr*2,_mrv],
+                 [_mr*3,_mrv],[_mr*3,-_mrv],[_mr*4,-_mrv],[_mr*4,_mrv],
+                 [_mr*5,_mrv],[_mr*5,-_mrv],[_mr*6,-_mrv],[_mr*6,0],
+                 [_mr*7,0]],
+
+              ],
+    }
+
+JJ = {   # Josphson Junction
+    'name'  : 'JJ',
+    'paths' : [ [[0,0],_gap,[-_rh,_rh],[_rh,-_rh],_gap,[_rh,_rh],[-_rh,-_rh],_gap,[0,0]] ],
+    }
 
 # Connection dots, lines
 _dotr = .075
@@ -369,18 +502,22 @@ BJT_NPN = {
     'base' : BJT,
     'shapes' : [ { 'shape' : 'arrow',
                    'start' : [_bjt_v,-_bjt_a],
-                   'end'   : [_bjt_emx,-_bjt_emy] }  ]
+                   'end'   : [_bjt_emx,-_bjt_emy],
+                   'headwidth' : 0.2
+                    }  ]
     }
 
 BJT_PNP = {
     'name'  : 'BJT_PNP',
     'base' : BJT,
     'shapes' : [ { 'shape' : 'arrow',
-                   'end'    : [_bjt_v,_bjt_a],
-                   'start'   : [_bjt_emx,_bjt_emy] } ],
+                   'end'   : [_bjt_v,_bjt_a],
+                   'start' : [_bjt_emx,_bjt_emy],
+                   'headwidth' : 0.2
+                    } ],
     'anchors' : { 'base'      : [0, 0],
-                  'emitter' : [_bjt_emx,_bjt_emy+_bjt_a],
-                  'collector'   : [_bjt_emx,-_bjt_emy-_bjt_a] }
+                  'emitter'   : [_bjt_emx,_bjt_emy+_bjt_a],
+                  'collector' : [_bjt_emx,-_bjt_emy-_bjt_a] }
      }
 
 BJT_NPN_C = {
@@ -399,6 +536,15 @@ BJT_PNP_C = {
                    'radius' : _bjt_r  }]
     }
 
+# BJT's with two collectors
+_bjt_2c_dy = -.25
+BJT_PNP_2C = {
+    'name' : 'BJT_PNP_2C',
+    'base' : BJT_PNP,
+    'paths' : [ [[_bjt_v,-_bjt_a-_bjt_2c_dy],[_bjt_emx,-_bjt_emy-_bjt_2c_dy]] ],
+    'anchors' : {'C2' : [_bjt_emx,-_bjt_emy-_bjt_2c_dy] }
+}
+
 # Inductor without spiraling
 _ind_w = .25
 _ind_shape_list = []
@@ -414,20 +560,9 @@ INDUCTOR = {
     'paths' : [ [[0,0],_gap,[1,0] ] ],
     'shapes' : _ind_shape_list }
 
-# Inductor with spiraling - "Prolate cycloid"
-# Magic numbers that work to look like an inductor spiral
-_a = .25
-_b = .6
-_t = _np.linspace(1.4,9.55*_np.pi,100)
-_x = _a*_t - _b*_np.sin(_t)
-_y = _a - _b * _np.cos(_t)
-_x = (_x - _x[0])  # Scale to about the right size
-_x = _x / _x[-1]
-_y = (_y - _y[0]) * .4
-_ind_path = _np.transpose(_np.vstack((_x,_y)))
 INDUCTOR2 = {
     'name'  : 'INDUCTOR2',
-    'paths' : [ _ind_path ]
+    'paths' : [ _cycloid(loops=4) ]
     }
 
 # Independent sources
@@ -494,6 +629,30 @@ SOURCE_CONT_I = {
                    'end'   : [.25,0] } ]
     }
 
+# Batteries
+_batw = _rh*.75
+_bat1 = _rh*1.5
+_bat2 = _rh*.75
+BAT_CELL = {
+    'name'  : 'BAT_CELL',
+    'paths' : [ [[0,_bat1],[0,-_bat1] ],
+                [[_batw,_bat2],[_batw,-_bat2]],
+                [[0,0],_gap,[_batw,0]]
+              ]
+    }
+
+BATTERY = {
+    'name'  : 'BATTERY',
+    'paths' : [ [[0,_bat1],[0,-_bat1] ],
+                [[_batw,_bat2],[_batw,-_bat2]],
+                [[_batw*2,_bat1],[_batw*2,-_bat1] ],
+                [[_batw*3,_bat2],[_batw*3,-_bat2]],
+                [[0,0],_gap,[_batw*3,0]]
+              ]
+    }
+
+
+
 # Meters
 METER_V = {
     'name'  : 'METER_V',
@@ -649,8 +808,145 @@ SWITCH_SPDT2_CLOSE = {
                    'arrow'  : 'cw' } ]
     }
 
+# Pushbuttons
+BUTTON = {
+    'name'  : 'BUTTON',
+    'paths'  : [ [ [0,0],_gap,[_sw_dot_r,.3],[1-_sw_dot_r,.3],_gap,[.5,.3],[.5,.5],_gap,[1,0] ] ],
+    'shapes' : [ { 'shape'  : 'circle',
+                   'center' : [_sw_dot_r,0],
+                   'radius' : _sw_dot_r },
+                 { 'shape'  : 'circle',
+                   'center' : [1-_sw_dot_r,0],
+                   'radius' : _sw_dot_r } ]
+    }
+
+BUTTON_NC = {
+    'name'  : 'BUTTON_NC',
+    'paths'  : [ [ [0,0],_gap,[_sw_dot_r,-_sw_dot_r-.05],[1-_sw_dot_r,-_sw_dot_r-.05],_gap,[.5,-_sw_dot_r-.05],[.5,_sw_dot_r+.15],_gap,[1,0] ] ],
+    'shapes' : [ { 'shape'  : 'circle',
+                   'center' : [_sw_dot_r,0],
+                   'radius' : _sw_dot_r },
+                 { 'shape'  : 'circle',
+                   'center' : [1-_sw_dot_r,0],
+                   'radius' : _sw_dot_r } ]
+    }
 
 
+# Speakers, bulbs, etc.
+_sph = .5
+SPEAKER = {
+    'name'  : 'SPEAKER',
+    'paths'  : [ [ [0,0],[_rh,0] ],
+                 [ [0,-_sph],[_rh,-_sph]]
+            ],
+    'drop'   : [0,-_sph],
+    'extend' : False,
+    'shapes' : [ { 'shape' : 'poly',
+                   'xy'    : [ [_rh,_sph/2],[_rh,-_sph*1.5],[_rh*2,-_sph*1.5],[_rh*2,_sph/2] ] },
+                 { 'shape' : 'poly',
+                   'xy'    : [ [_rh*2,_sph/2], [_rh*3.5,_sph*1.25],[_rh*3.5,-_sph*2.25],[_rh*2,-_sph*1.5] ],
+                   'closed' : False } ]
+    }
+
+_a = .25
+_b = .7
+_t = _np.linspace(1.4,3.6*_np.pi,100)
+_x = _a*_t - _b*_np.sin(_t)
+_y = _a - _b * _np.cos(_t)
+_x = (_x - _x[0])  # Scale to about the right size
+_x = _x / _x[-1]
+_y = (_y - _y[0]) * .25
+_lamp_path = _np.transpose(_np.vstack((_x,_y)))
+LAMP = {
+    'name'  : 'LAMP',
+    'base' : SOURCE,
+    'paths': [_lamp_path]
+    }
+
+
+def transformer(t1=4, t2=4, core=True, ltaps=None, rtaps=None, loop=False):
+    ''' Transformer element
+        t1 = Turns on primary (left) side
+        t2 = Turns on secondary (right) side
+        core = Draw the core (parallel lines) [default=True]
+        ltaps = Dictionary of name:position pairs, position is the turn number from the top to tap
+                Each tap defines an anchor point but does not draw anything.
+        rtaps = Same as ltaps, on right side
+        loop = Use spiral/cycloid (loopy) style [default=False]
+    '''
+    # Set initial parameters
+    ind_w = .4
+    lbot = 0
+    ltop = t1*ind_w
+    right_ofst = t1*ind_w/2    
+    rtop = (ltop+lbot)/2 + t2*ind_w/2
+    rbot = (ltop+lbot)/2 - t2*ind_w/2
+
+    element = {'name' : 'TRANSFORMER',
+               'extend' : False,
+               'paths' : [] }
+
+    # Adjust for loops or core
+    ind_gap = .75    
+    if loop:
+        ind_gap = ind_gap + .4
+    if core:
+        ind_gap = ind_gap + .25
+
+    ltapx = 0
+    rtapx = ind_gap        
+
+    # Draw coils
+    if loop:
+        c1 = _cycloid(loops=t1, ofst=(0,0), norm=False, vertical=True)
+        c2 = _cycloid(loops=t2, ofst=(ind_gap,-rtop+ltop), norm=False, flip=True, vertical=True) 
+        ltapx = min([i[0] for i in c1])
+        rtapx = max([i[0] for i in c2])
+        ltop = c1[-1][1]
+        rtop = c2[-1][1]
+        element['paths'].extend([c1, c2])
+    else:
+        shapes = []
+        for i in range(t1):
+            shapes.append( {'shape':'arc',
+                            'center':[0, ltop-(i*ind_w+ind_w/2)],
+                            'theta1' : 270,
+                            'theta2' : 90,
+                            'width'  : ind_w,
+                            'height' : ind_w } )    
+        for i in range(t2):
+            shapes.append( {'shape':'arc',
+                            'center':[ind_gap, rtop-(i*ind_w+ind_w/2)],
+                            'theta1' : 90,
+                            'theta2' : 270,
+                            'width' : ind_w,
+                            'height' : ind_w })
+        element['shapes'] = shapes
+
+    # Add the core
+    if core:
+        top = max(ltop, rtop)
+        bot = min(lbot, rbot)
+        center = ind_gap/2
+        core_w = ind_gap/10
+        element['paths'].extend( [[[center-core_w, top],[center-core_w, bot]],
+                                  [[center+core_w, top],[center+core_w, bot]]] )
+        
+    anchors = {'p1' : [0,ltop], 
+               'p2' : [0,lbot],
+               's1' : [ind_gap, rtop],
+               's2' : [ind_gap, rbot]}
+
+    if ltaps:
+        for name, pos in ltaps.items():
+            anchors[name] = [ltapx, ltop - pos * ind_w]
+    if rtaps:
+        for name, pos in rtaps.items():
+            anchors[name] = [rtapx, rtop - pos * ind_w]
+
+    element['anchors'] = anchors
+    return element
+    
 
 def blackbox( w, h, linputs=None, rinputs=None, tinputs=None, binputs=None, mainlabel='', leadlen=0.5 ):
     ''' Generate a black-box element consisting of rectangle with inputs on each side.
