@@ -53,7 +53,11 @@ def _merge_elements( elm_def ):
         elm_def['paths']   = elm_def.get('paths',[] ) + base_elm.get( 'paths',  [] )
         elm_def['labels']  = elm_def.get('labels',[]) + base_elm.get( 'labels', [] )
         elm_def['shapes']  = elm_def.get('shapes',[]) + base_elm.get( 'shapes', [] )
-        elm_def['anchors'] = dict(list(elm_def.get('anchors',{}).items()) + list(base_elm.get( 'anchors', {} ).items()))
+        
+        # New anchors with same name should overwrite old
+        d = base_elm.get('anchors',{}).copy()
+        d.update(elm_def.get('anchors',{}))
+        elm_def['anchors'] = d
         
         for i in base_elm:
             # Everything that's not a list. New key/values overwrite old.
@@ -64,7 +68,7 @@ def _merge_elements( elm_def ):
 
 
 #--------------------------------------------------------------------
-def group_elements( drawing, anchors={} ):
+def group_elements( drawing, anchors=None ):
     ''' Combine all elements in a drawing into a single element that can be added to
         another drawing. Returns an element definition.
 
@@ -90,7 +94,8 @@ def group_elements( drawing, anchors={} ):
                 s['end']   = elm.translate( s['start'] - elm.ofst )
             new_elm['shapes'].append( s )
 
-        new_elm['anchors'] = anchors
+        if anchors is not None:
+            new_elm['anchors'] = anchors
 
         for label, loc, align, size in elm.strs:
             loc = elm.translate( np.array(loc) )
@@ -101,12 +106,11 @@ def group_elements( drawing, anchors={} ):
     return new_elm
 
 
-
 #--------------------------------------------------------------------
 # Main drawing class
 #--------------------------------------------------------------------
-class Drawing():
-    def __init__(self, unit=3.0, inches_per_unit=0.5, txtofst=0.1, fontsize=16, font='sans-serif', color='black'):
+class Drawing(object):
+    def __init__(self, unit=3.0, inches_per_unit=0.5, txtofst=0.1, fontsize=16, font='sans-serif', color='black', lw=1.5):
         """ Set up a new circuit drawing.
     unit : Full length of a resistor element in matplotlib plot units.
            Inner portion of resistor is length 1.
@@ -114,6 +118,8 @@ class Drawing():
     txtofst  : Default distance from element to text label
     fontsize : Default font size
     font     : matplotlib font-family
+    color    : default color for elements
+    lw       : default line width
     """
 
         # Default values
@@ -123,6 +129,7 @@ class Drawing():
         self.fontsize = fontsize
         self.font = font
         self.color = color
+        self.lw = lw
 
         # State variables
         self.here = np.array([0,0])
@@ -191,23 +198,32 @@ Other:
             self.here, self.theta = self._state.pop()
 
 
-    def draw(self, showframe=False):
+    def draw(self, ax=None, showframe=False, showplot=True):
         """ Draw the diagram.
-            shwoframe : Show the plot frame/axis. Useful for debugging.
+            ax : matplotlib axis to draw to.
+            showframe : Show the plot frame/axis. Useful for debugging.
+            showplot : Show the plot in matplotlib window in non-interactive mode.
         """
 
         mpl.rcParams['font.size'] = self.fontsize
         mpl.rcParams['font.family'] = self.font
 
-        fig, ax = plt.subplots()
+        if ax==None:
+            fig, ax = plt.subplots()
+        else:
+            fig = ax.get_figure()
+
         for e in self._elm_list:
             e.draw(ax)
 
         ax.autoscale_view(True)  # This autoscales all the shapes too
+        # NOTE: arrows don't seem to be included in autoscale!
         xlim = np.array(ax.get_xlim())
         ylim = np.array(ax.get_ylim())
-        xlim = xlim + xlim * .05  # Add a 5% border to pick up lost pixels
-        ylim = ylim + ylim * .05
+        xlim[0] = xlim[0]-.1   # Add a .1 unit border to pick up lost pixels
+        ylim[0] = ylim[0]-.1
+        xlim[1] = xlim[1]+.1
+        ylim[1] = ylim[1]+.1
         ax.set_xlim( xlim )
         ax.set_ylim( ylim )
         w = xlim[1]-xlim[0]
@@ -220,7 +236,7 @@ Other:
 
         self.ax = ax
         self.fig = ax.get_figure()
-        if not plt.isinteractive():
+        if not plt.isinteractive() and showplot:
             plt.show()
 
         # Grow the figure size so that elements are always the same
@@ -228,7 +244,6 @@ Other:
         ax.get_figure().set_size_inches( self.inches_per_unit*w, self.inches_per_unit*h )
 
 
-
     def save(self, fname, transparent=True, dpi=72 ):
         """ Save figure to file.
             fname : filename to save.
@@ -306,7 +321,7 @@ Other:
 #--------------------------------------------------------------------
 # Circuit element class
 #--------------------------------------------------------------------
-class Element():
+class Element(object):
     def __init__(self, elm_def, drawing, **kwargs):
         """ Initialize the element.
 elm_def  : element definition dictionary (see below)
@@ -346,6 +361,7 @@ Other:
     move_cur : move the cursor after drawing. Default=True.
     color    : matplotlib color for element. e.g. 'red', '#34a4e6', (.8,0,.8)
     ls       : line style. same as matplotlib: ['-', ':', '--']
+    lw       : line width. same as matplotlib. Default=1
     """
         # Flatten element def with base elements
         self.defn   = elm_def.copy()
@@ -357,6 +373,7 @@ Other:
         self.shapes = self.defn.get( 'shapes', [] )
         self.color  = kwargs.get( 'color', self.defn.get('color', drawing.color ) )
         self.ls     = kwargs.get( 'ls', self.defn.get('ls', '-') )
+        self.lw     = kwargs.get( 'lw', self.defn.get('lw', drawing.lw) )
         totlen      = kwargs.get( 'l', drawing.unit   )
 
         # Determine theta angle of element
@@ -696,7 +713,7 @@ Other:
             showframe : Draw the axis frame. Useful for debugging.
         """
         for path in self.paths:
-            ax.plot( path[:,0], path[:,1], color=self.color, lw=1,
+            ax.plot( path[:,0], path[:,1], color=self.color, lw=self.lw,
                      solid_capstyle='round', ls=self.ls )
 
         for s in self.shapes:
@@ -707,7 +724,7 @@ Other:
                 fill = s.get('fill', False)
                 fillcolor = s.get('fillcolor', self.color)
                 circ = plt.Circle( xy=xy, radius=rad, ec=self.color,
-                                   fc=fillcolor, fill=fill, zorder=3 )
+                                   fc=fillcolor, fill=fill, zorder=3, lw=self.lw )
                 ax.add_patch(circ)
             elif s.get('shape') == 'poly':
                 xy = np.array(s.get('xy', [[0,0]]))
@@ -716,7 +733,7 @@ Other:
                 fill = s.get('fill', False)
                 fillcolor = s.get('fillcolor', self.color)
                 poly = plt.Polygon( xy=xy, closed=closed, ec=self.color,
-                                    fc=fillcolor, fill=fill, zorder=3 )
+                                    fc=fillcolor, fill=fill, zorder=3, lw=self.lw )
                 ax.add_patch(poly)
             elif s.get('shape') == 'arc':
                 xy = np.array(s.get('center', [0,0] ) )
@@ -730,7 +747,7 @@ Other:
 
                 angle = s.get('angle', self.theta )
                 arc = Arc( xy, width=w, height=h, theta1=th1,
-                           theta2=th2, angle=angle, color=self.color )
+                           theta2=th2, angle=angle, color=self.color, lw=self.lw )
                 ax.add_patch(arc)
 
                 # Add an arrowhead to the arc
@@ -767,7 +784,7 @@ Other:
                 hl   = s.get('headlength', .2 )
                 ax.arrow(start[0], start[1], end[0]-start[0], end[1]-start[1],
                          head_width=hw, head_length=hl,
-                         length_includes_head=True, color=self.color)
+                         length_includes_head=True, color=self.color, lw=self.lw)
 
         for label, loc, align, size in self.strs:
             loc = self.translate( np.array(loc) )
